function [tab_final]=RotDif_demo(freq,r1,r2,r3,NH,reslist,output,mc_max,niter,guessMC_ax,guessMC_full)
%------------------------------------------------------------------
%   ow-2002-University of Maryland
%   based on the program r2r1fit from david fushman
%
%
%   Optimization of isotropic, axially symmetric and fully anisotropic model
%   
%   freq : frequency 
%   r1,r2,r3 : R1, R2, NOE
%   output : name of output file
%
%   guessMC_ax : 1*8 array containing boundary conditions to simulate starting 
%                 guess prior to data fitting  (axially symmetric case),and boundary conditions
%                  (used in the fitting procedure for axially symmetric case) 
%   [lower bound for TAUx,upper bound for TAUX,lower bound for D/D,.....alpha,....beta]
%
%
%
%	CALL: RotDif(freq,r1,r2,r3,NH,reslist,output,mc_max,niter,guessMC_ax,guessMC_full)
%	      
%   mc_max : number of MC step for error calculation (default=1000)
%   niter : number of simulated starting guess prior to optimization(default=50)
%
%   guessMC_ax=[lower bound_TAUx,upper bound_TAUx,lb_D/D,ub_D/D,lb_alpha,ub_alpha,lb_beta,ub_beta]
%   bound_ax=[lower bound_TAUx,lb_D/D,lb_alpha,lb_beta,ub_TAUx,ub_D/D,ub_alpha,ub_beta]
%
%   bound_full=same as bound_ax for Dxx, Dyy, Dzz, alpha, beta, gamma ----> 1*12 array
%   guessMC_full= same as guessMC_ax for Dxx, Dyy, Dzz, alpha, beta, gamma ----> 1*12 array
%
%
%
%------------------------------------------------------------------
format compact
%--------check input, set some params-------


if nargin<11, guessMC_full=[0.5 12.0 0.5 12.0 0.5 12.0 0.0 pi 0.0 pi 0.0 pi];	end
if nargin<10,  guessMC_ax=[1.0 15.0 1.0 1.8 0.0 pi 0.0 pi]; end
if nargin<9, niter=50;	end		%default: number of initial values for MC estimation
if nargin<8, mc_max=1000;	end		%default: number of MC steps for statistical errors 


wN=freq*(2710.5/26750)*2*pi/1000;
wNfull=freq*0.1013756*2*pi*1.e06;
flag_case=0;     %check for axially symmetric or fully anisotropic case

bound_full=guessMC_full;
bound_ax=guessMC_ax;

%----- save data -------
fid_out=fopen(output,'w');



%------------check output-------------------
if nargout>=4, kmap=1; else kmap=0; end
sdzres=NaN*ones(1,4);
%----------prepare working set of data-----------
[ratio,sigma,vcoor,rlist]=r2r1prep(r2,r1,NH,reslist,r3);
nres=length(ratio);
dfiso=nres-1;     %degree of freedom isotropic case
df=nres-4;        %degree of freedom axially symmetric
df_full=nres-6;   %degree of freedom fully anisotropic
fprintf('\n\n');
disp([num2str(nres),' residues selected']);
fprintf('\n\n');

%------ save in output file --------
fprintf(fid_out,'%3.0d residues selected\n',nres);










%=======================================================================================
%                                                                                      =
%===============================  FULLY ANISOTROPIC CASE ===============================
%                                                                                      =
%=======================================================================================


start=ones(niter,6); 



%------ Array of random initial guess to search the best solution-----
for i=1:niter
    start(:,1)=guessMC_full(1)+(guessMC_full(2)-guessMC_full(1))*rand(niter,1);
    start(:,2)=guessMC_full(3)+(guessMC_full(4)-guessMC_full(3))*rand(niter,1);
    start(:,3)=guessMC_full(5)+(guessMC_full(6)-guessMC_full(5))*rand(niter,1);
    start(:,4)=guessMC_full(7)+(guessMC_full(8)-guessMC_full(7))*rand(niter,1);
    start(:,5)=guessMC_full(9)+(guessMC_full(10)-guessMC_full(9))*rand(niter,1);
    start(:,6)=guessMC_full(11)+(guessMC_full(12)-guessMC_full(11))*rand(niter,1); 
end




%----------- calculate difference between calculated and exp data -----
mapendlsq=ones(niter,7);

%----- bounding conditions in the fitting procedure-------
lb=[bound_full(1) bound_full(3) bound_full(5) bound_full(7) bound_full(9) bound_full(11)];   %lower bound for Dxx Dyy Dzz alpha beta gamma
ub=[bound_full(2) bound_full(4) bound_full(6) bound_full(8) bound_full(10) bound_full(12)];   %upper bound for Dxx Dyy Dzz alpha beta gamma
    
options_full=optimset('TolX',1.e-15,'TolFun',1e-15,'MaxIter',8000,'maxFunEvals',20000,'LargeScale','on');
fprintf('\n\n');    
fprintf('********************************************************************************************\n');
fprintf('****************  Starting Monte Carlo simulation with %0.f initial values   *****************\n',niter);
fprintf('****************               for fully anisotropic model                 *****************\n');
fprintf('********************************************************************************************\n\n');
tic

fprintf('Calculation in progress\n\n');
fprintf('0%%');
for i=1:niter-6
    fprintf(' ');
end
fprintf('100%% \n');


%----begin the MC search-------
for i=1:niter
     x=lsqnonlin(@rho_full2,start(i,:),lb,ub,options_full,ratio,sigma,vcoor,wNfull); %---- LVM algorithm----
     errlsq=rho_full(x,ratio,sigma,vcoor,wNfull);
     
     %------ calc progress ---------
   
    fprintf('>');
     
    
    %---- array of results with LM-----
    mapendlsq(i,1)=x(1);
    mapendlsq(i,2)=x(2);
    mapendlsq(i,3)=x(3);
    mapendlsq(i,4)=x(4);
    mapendlsq(i,5)=x(5);
    mapendlsq(i,6)=x(6);
    mapendlsq(i,7)=errlsq;
    
end

fprintf('\n\n');
toc





%---reorder matrix of results to pick the best target function-----
%---- with the convention  Dzz>Dyy>Dxx ------------Initial valuesInitial values----------------

sortlsq_full=sortrows(mapendlsq,7);
j=1;
u=0;
lim=sortlsq_full(1,7);

for i=1:niter
    if((sortlsq_full(i,7)-lim)<=10^-5) %count number of lowest target functions
        u=u+1;
    end
end

der=ones(u,7);
for i=1:niter
    if ((sortlsq_full(i,7)-lim)<=10^-5); %build array containing only the best target functions
        der(j,:)=sortlsq_full(i,:);
        
    end
    j=j+1;
end
sol=ones(1,7);
sortder=sortrows(der,1);
for i=1:u
    if ((sortder(i,1)<sortder(i,2))&(sortder(i,2)<sortder(i,3))); %sort so as to obey convention Dzz>Dyy>Dxx
        sol(1,:)=sortder(i,:);
    
    end
end




%------ prepare report-------
zres=sol;

D2tau_lsq=[10^2/(6*zres(1)) 10^2/(6*zres(2)) 10^2/(6*zres(3))]; %calculates TAUx TAUy TAUz for LVM

tauclsq=D2tc(zres(1),zres(2),zres(3));                          %calculates TAUc

Anlsq=aniso(zres(1),zres(2),zres(3));                           %Anisotropy

rhomblsq=rhomb(zres(1),zres(2),zres(3));                        %Rhombicity

[DztoDz,DztoDy,DztoDx]=rap(zres(1),zres(2),zres(3));            %calculates ratio Dzz/Dzz Dzz/Dyy Dzz/Dxx







%------ Confidence limits ------------------------------


    
full_par3=[zres(1),zres(2),zres(3),zres(4),zres(5),zres(6)];

matrix_full=rotation_matrix(zres(4),zres(5),zres(6));
dR_dalpha=matrix_dalpha(zres(4),zres(5),zres(6));
dR_dbeta=matrix_dbeta(zres(4),zres(5),zres(6));
dR_dgamma=matrix_dgamma(zres(4),zres(5),zres(6));

mat_full3=invcovar_fullax_angle(full_par3,sigma,matrix_full,dR_dalpha,dR_dbeta,dR_dgamma,vcoor,wNfull);



[me_full3,sd_full3]=r2r1conf(full_par3,zres(7)/df_full,mat_full3,mc_max);
aniso2=aniso(me_full3(1),me_full3(2),me_full3(3));
error_aniso=aniso2*sqrt((sd_full3(3)/me_full3(3))^2-(sd_full3(2)/me_full3(2))^2+(sd_full3(1)/me_full3(1))^2);





tab_final=[zres(1),sd_full3(1)/1e7,zres(2),sd_full3(2)/1e7,zres(3),sd_full3(3)/1e7,zres(4)*180.0/pi,sd_full3(4)*180.0/pi,zres(5)*180.0/pi,sd_full3(5)*180.0/pi,zres(6)*180.0/pi,sd_full3(6)*180.0/pi]


%----- visualise points ---------------------




fclose(fid_out);
return

%===================================================================






