function [tab_final]=RotDif_demo_ax(freq,r1,r2,r3,NH,reslist,output,alg,mc_max,niter,guessMC_ax,guessMC_full)
%------------------------------------------------------------------
%   ow-2002-University of Maryland
%   based on the program r2r1fit from david fushman
%
%
%   Optimization of isotropic, axially symmetric and fully anisotropic model
%   
%   freq : frequency 
%   r1,r2,r3 : R1, R2, NOE
%   output : name of output file
%
%   guessMC_ax : 1*8 array containing boundary conditions to simulate starting 
%                 guess prior to data fitting  (axially symmetric case),and boundary conditions
%                  (used in the fitting procedure for axially symmetric case) 
%   [lower bound for TAUx,upper bound for TAUX,lower bound for D/D,.....alpha,....beta]
%
%
%
%	CALL: RotDif(freq,r1,r2,r3,NH,reslist,output,mc_max,niter,guessMC_ax,guessMC_full)
%	      
%   mc_max : number of MC step for error calculation (default=1000)
%   niter : number of simulated starting guess prior to optimization(default=50)
%
%   guessMC_ax=[lower bound_TAUx,upper bound_TAUx,lb_D/D,ub_D/D,lb_alpha,ub_alpha,lb_beta,ub_beta]
%   bound_ax=[lower bound_TAUx,lb_D/D,lb_alpha,lb_beta,ub_TAUx,ub_D/D,ub_alpha,ub_beta]
%
%   bound_full=same as bound_ax for Dxx, Dyy, Dzz, alpha, beta, gamma ----> 1*12 array
%   guessMC_full= same as guessMC_ax for Dxx, Dyy, Dzz, alpha, beta, gamma ----> 1*12 array
%
%
%
%------------------------------------------------------------------
format compact
%--------check input, set some params-------


if nargin<12, guessMC_full=[0.5 12.0 0.5 12.0 0.5 12.0 0.0 pi 0.0 pi 0.0 pi];	end
if nargin<12,  guessMC_ax=[1.0 15.0 1.0 1.8 0.0 pi 0.0 pi]; end
if nargin<10, niter=50;	end		%default: number of initial values for MC estimation
if nargin<9, mc_max=1000;	end		%default: number of MC steps for statistical errors 
if nargin<8, alg=1;	end

wN=freq*(2710.5/26750)*2*pi/1000;
wNfull=freq*0.1013756*2*pi*1.e06;
flag_case=0;     %check for axially symmetric or fully anisotropic case
flag_visu=0;

bound_full=guessMC_full;
bound_ax=guessMC_ax;

%----- save data -------
fid_out=fopen(output,'w');



%------------check output-------------------
if nargout>=4, kmap=1; else kmap=0; end
sdzres=NaN*ones(1,4);
%----------prepare working set of data-----------
[ratio,sigma,vcoor,rlist]=r2r1prep(r2,r1,NH,reslist,r3);
nres=length(ratio);
dfiso=nres-1;     %degree of freedom isotropic case
df=nres-4;        %degree of freedom axially symmetric
df_full=nres-6;   %degree of freedom fully anisotropic
fprintf('\n\n');

fprintf('\n\n');





%=======================================================================================
%                                                                                      =
% =======================  AXIALLY SYMMETRIC CASE ======================================
%                                                                                      =
%=======================================================================================




%---------------- Monte carlo distribution to create a set of initial values --------------

 
count=[1:1:niter];  %-- count number of synthetic data, used for drawing
trialrad=ones(niter,4);

%------ Array of random initial guess to search the best solution-----
for i=1:niter
    trialrad(:,1)=guessMC_ax(1)+(guessMC_ax(2)-guessMC_ax(1))*rand(niter,1);
    trialrad(:,2)=guessMC_ax(3)+(guessMC_ax(4)-guessMC_ax(3))*rand(niter,1);
    trialrad(:,3)=guessMC_ax(5)+(guessMC_ax(6)-guessMC_ax(5))*rand(niter,1);
    trialrad(:,4)=guessMC_ax(7)+(guessMC_ax(8)-guessMC_ax(7))*rand(niter,1);
end

fprintf('Search Range\n\n');
fprintf('%5.1f < TAUx  < %5.1f\n',guessMC_ax(1),guessMC_ax(2));
fprintf('%5.1f < D/D   < %5.1f\n',guessMC_ax(3),guessMC_ax(4));
fprintf('%5.1f < alpha < %5.1f\n',guessMC_ax(5)*180.0/pi,guessMC_ax(6)*180.0/pi);
fprintf('%5.1f < beta  < %5.1f\n',guessMC_ax(7)*180.0/pi,guessMC_ax(8)*180.0/pi);
fprintf('\n\n');


%------------ angles in degree-----------------

for i=1:niter
    trialdeg(:,1)=trialrad(:,1);
    trialdeg(:,2)=trialrad(:,2);
    trialdeg(:,3)=trialrad(:,3)*180.0/pi;
    trialdeg(:,4)=trialrad(:,4)*180.0/pi;
end





%----------- calculate difference between calculated and exp data -----

reslsq=ones(niter,5);  %for LVM algorithm


%----- bounding conditions in the fitting procedure-------

lb=[bound_ax(1) bound_ax(3) bound_ax(5) bound_ax(7)]; %lower bound for TAUc D/D alpha beta
ub=[bound_ax(2) bound_ax(4) bound_ax(6) bound_ax(8)];   %upper bound for TAUc D/D alpha beta


%----- Fitting procedure ---------------------------------
if (alg==1),    
    options=optimset('TolX',1.e-15,'TolFun',1e-15,'MaxIter',8000,'maxFunEvals',20000,'LargeScale','on','LevenbergMarquardt','on');
else
    options=optimset('TolX',1.e-15,'TolFun',1e-15,'MaxIter',8000,'maxFunEvals',20000,'LargeScale','on');
end

fprintf('\n\n');  
fprintf('*********************************************************************************************\n');
fprintf('****************  Starting Monte Carlo simulation with %.0f initial values    ****************\n',niter);
fprintf('****************               for axially symmetric model                  ****************\n');
fprintf('*********************************************************************************************\n\n');
tic

fprintf('Calculation in progress\n\n');
fprintf('0%%');
for i=1:niter-6
    fprintf(' ');
end
fprintf('100%% \n');
    

%----begin the MC search-------
for i=1:niter
    x_ax=lsqnonlin(@rho2,trialrad(i,:),lb,ub,options,ratio,sigma,vcoor,wN); %---- LVM algorithm----- 
    errlsq_ax=rho(ratio,sigma,vcoor,wN,x_ax);          % Chi^2 for LVM algorithm
    
    
    %------ calc progress ---------
   
    fprintf('>');
         
    %---- array of results with Levenberg Marquardt-----
    reslsq(i,1)=x_ax(1);
    reslsq(i,2)=x_ax(2);
    reslsq(i,3)=x_ax(3);
    reslsq(i,4)=x_ax(4);
    reslsq(i,5)=errlsq_ax;
end


toc


%---reorder matrix of results to pick the best target function ---------
%---- in radians ----
sortlsqrad=sortrows(reslsq,5);


%---- calculate TAUc for the best fit --------
Tauclsq=tx2tc(sortlsqrad(1,1),sortlsqrad(1,2));

%------ prepare report --------
zres_ax=[sortlsqrad(1,:)];

%----- calculate Dxx, Dyy, Dzz -------
endDx=calcDx(zres_ax(1));
endDz=calc_Dz(zres_ax(2),zres_ax(1));





%--------------standard errors in TAU & D/D using inverse comatrix procedure -------------


   invCov=r2r1covMC2(zres_ax(1:4),ratio,sigma,vcoor,wN);
   [me_mc,sd_mc]=r2r1mc(zres_ax(1:4),zres_ax(5)/df,invCov,mc_max,1);
   tc_mc=tx2tc(me_mc(1),me_mc(2));      
   sd_tc_mc=tc_mc*sqrt((sd_mc(1)/me_mc(1))^2+(sd_mc(2)/(me_mc(2)+2))^2);    % error in Tauc
   sdzres(1:4)=sd_mc;
   
   endDx2=calcDx(me_mc(1));
   endDz2=(calc_Dz(me_mc(2),me_mc(1)));
   errDx=(endDx2/me_mc(1))*sd_mc(1);                                %error in Dxx, Dyy
   errDz=endDz2*sqrt((sd_mc(2)/me_mc(2))^2+(sd_mc(1)/me_mc(1))^2);         %error in Dz
   
   
   
   
   tab_final=[Tauclsq,sd_tc_mc,zres_ax(2),sd_mc(2),zres_ax(3)*180.0/pi,sd_mc(3)*180.0/pi,zres_ax(4)*180./pi,sd_mc(4)*180.0/pi]




fprintf('\n\n');
fclose(fid_out);
return

%===================================================================






